from __future__ import absolute_import 

from sctokenizer.assets.c_keywords import *
from sctokenizer.assets.cpp_keywords import *
from sctokenizer.assets.java_keywords import *
from sctokenizer.assets.python_keywords import *
