c_keyword_set = {
    'auto',
    'break',
    'case',
    'char',
    'const',
    'continue',
    'default',
    'define',
    'do',
    'double',
    'elif',
    'else',
    'endif',
    'enum',
    'error',
    'extern',
    'float',
    'for',
    'goto',
    'if',
    'ifdef',
    'ifndef',
    'include',
    'inline',
    'int',
    'line',
    'long',
    'noalias',
    'pragma',
    'register',
    'restrict',
    'return',
    'short',
    'signed',
    'sizeof',
    'static',
    'struct',
    'switch',
    'typedef',
    'undef',
    'union',
    'unsigned',
    'void',
    'volatile',
    'while'
}